<?php

namespace App\Console\Commands;

use App\Repositories\Enums\CheckStatusEnum;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\Navigation\TypeEnum;
use App\Repositories\Enums\Navigation\VisitTypeEnum;
use App\Repositories\Enums\Qunali\EventEnum;
use App\Repositories\Models\Area\Area;
use App\Repositories\Models\Area\Projects;
use App\Repositories\Models\Base\Resource;
use App\Repositories\Models\Course\Chapter;
use App\Repositories\Models\Course\Course;
use App\Repositories\Models\Course\Video;
use App\Repositories\Models\Navigation\Category;
use App\Repositories\Models\Navigation\UserOrganization;
use App\Repositories\Models\Navigation\Website;
use App\Repositories\Models\Qunali\Organization;
use App\Repositories\Models\Qunali\Point;
use App\Repositories\Models\Qunali\PointEvent;
use App\Repositories\Models\TCM\Combinations;
use App\Repositories\Models\TCM\Prescription;
use App\Repositories\Models\TCM\Recipels;
use App\Repositories\Models\User\User;
use App\Services\CombinationService;
use App\Services\TencentMapService;
use Carbon\Carbon;
use Illuminate\Console\Command;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

/**
 * Class PresenterCommand
 * @package Prettus\Repository\Generators\Commands
 * @author Anderson Andrade <contato@andersonandra.de>
 */
class TestCommand extends Command
{
    /**
     * The name of command.
     *
     * @var string
     */
    protected $name = 'mead:test';
    /**
     * The description of command.
     *
     * @var string
     */
    protected $description = 'Create a new course.';
    /**
     * The type of class being generated.
     *
     * @var string
     */
    protected $type = 'course';

    /**
     * Execute the command.
     *
     * @return void
     * @see fire()
     */
    public function handle()
    {
//        $areas = Area::query()->where('admin_id', 1)->where('shop_id', 0)->get();
//        foreach ($areas as $area) {
//            $area->shop_id = Projects::query()->where('id', $area->project_id)->value('shop_id');
//            $area->save();
//        }

//        $users = User::query()->get();
//
//        foreach ($users as $user) {
//            UserOrganization::query()->updateOrCreate(['organization_id' => $user->organization_id, 'user_id' => $user->id], ['role' => $user->role_id, 'status' => ModelStatusEnum::OK]);
//        }

//        $this->geocoder();

//        $this->loadData();
        $this->randomData();

        $this->line('ok');
    }

    public function randomData()
    {
//        where('check_status', CheckStatusEnum::OK)
        $points = Point::query()->status()->get();
        $day = date('Y-m-d');
        foreach ($points as $point) {
            $good = rand(0, 20);
            $view = rand(0, 100);

            $point->good_nums = $good;
            $point->check_status = CheckStatusEnum::OK;
            $point->bad_nums = 0;
            $point->report_nums = 0;
            $point->view_nums = $view;
//            $point->view_nums = $point->view_nums + $view;

            $intData = [];
            $time = Carbon::now()->toDateTimeString();
            for ($i = 0; $i < $good; $i++) {
                $intData[] = [
                    'user_id' => 0,
                    'day' => $day,
                    'type' => EventEnum::GOOD,
                    'point_id' => $point->id,
                    'status' => ModelStatusEnum::OK,
                    'created_at' => $time,
                    'updated_at' => $time,
                ];
            }
            $point->save();
            PointEvent::query()->insert($intData);
        }
    }

    public function geocoder()
    {
        $lat = 34.799812;
        $lng = 113.797962;
        $data = (new TencentMapService())->geocoder($lat, $lng);
    }

    /**
     * 加载远程数据
     * Author: Mead
     */
    public function loadData()
    {
        $url = 'https://cdn.digitalcnzz.com/static/json/virus-cabin.json';
        $client = new \GuzzleHttp\Client();
        $data = $client->request('get', $url, [
            'headers' => [
                'Accept' => 'application/json',
            ]
        ])->getBody()->getContents();
        $data = js2php($data);
        if ($data['code'] == 0) {
            $lists = $data['data'];

            foreach ($lists as $list) {
                foreach ($list as $item) {
                    Point::query()->create([
                        'organization_id' => 1,
                        'name' => $item['name'],
                        'slug' => Str::random(8),
                        'time' => $item['workTime'],
                        'area_name' => $item['areaId'],
                        'street_name' => $item['streetId'],
                        'address' => $item['address'],
                        'lat' => $item['lat'],
                        'lng' => $item['lng'],
                        'source_id' => $item['id'],
                    ]);
                }
            }

//            Organization::query()->updateOrCreate([
//                'name' => '核酸检测地图',
//            ], [
//                'slug' => Str::random(8),
//            ]);
        }


        $url = 'https://safetrip.zhengzhou.gov.cn:20021/disncov-gateway/disncov-common-service/areaList';
        $client = new \GuzzleHttp\Client();
        $data = $client->request('get', $url, [
            'headers' => [
                'Accept' => 'application/json',
            ]
        ])->getBody()->getContents();
        $data = js2php($data);
        if ($data['code'] == 0) {
            $lists = $data['data'];

            foreach ($lists as $list) {
                Point::query()->where('area_name', $list['value'])->update(['area_name' => $list['label']]);
                foreach ($list['children'] as $item) {
                    Point::query()->where('street_name', $item['value'])->update(['street_name' => $item['label']]);
                }
            }
        }


    }

    public function sort()
    {
        $organization_id = 46;
        $categories = Category::query()->where('id', '<>', 294)->where('pid', 0)->where('organization_id', $organization_id)->orderBy('id')->pluck('id');
        $nums = count($categories);
        //一级
        foreach ($categories as $cid) {


            $ids = Category::query()->where('organization_id', $organization_id)->where('pid', $cid)->orderByDesc('id')->pluck('id');
            $nums_t = count($ids);
            foreach ($ids as $id) {
                Category::query()->where('id', $id)->update(['sort' => $nums_t]);
                $nums_t--;
            }

        }

    }

    public function import()
    {
        $disk = 'public';
        $path = 'website.xlsx';
        $organization_id = 46;
//        $organization_id = 45;

        $config = ['path' => ''];
        $excel = new \Vtiful\Kernel\Excel($config);
        $excel->openFile(base_path('public/' . $path))
            ->openSheet()
            ->setSkipRows(1);
        $i = 1;
        $header = [
            '序号',
            '一级分类',
            '二级分类',
            '网站名称',
            '网址',
            '网站简介',
            '网站图标地址',
        ];
        $rows = [];
        $categories = [];
        while (($row = $excel->nextRow()) !== NULL) {
            $rows[] = $row;

            list(
                $id,
                $category_one,
                $category_two,
                $name,
                $url,
                $des,
                $logo_url,
                ) = $row;

            $img_type = last(str2arr($logo_url, '.'));
            $filepath = null;
            if (!empty($logo_url)) {
                try {
                    $client = new \GuzzleHttp\Client();
                    $data = $client->request('get', $logo_url)->getBody()->getContents();
                    $filepath = 'websites' . DIRECTORY_SEPARATOR . $name . '.' . $img_type;
                    Storage::disk('public')->put($filepath, $data);
                } catch (\Exception $e) {
                    $filepath = null;
                }
            }

            $one = Category::query()->firstOrCreate([
                'name' => $category_one,
                'pid' => 0,
                'type' => TypeEnum::ORGANIZATION,
                'organization_id' => $organization_id,
            ], [
                'visit_type' => VisitTypeEnum::COMMON,
                'status' => ModelStatusEnum::OK
            ]);

            $two = Category::query()->firstOrCreate([
                'name' => $category_two,
                'pid' => $one->id,
                'type' => TypeEnum::ORGANIZATION,
                'organization_id' => $organization_id,
            ], [
                'visit_type' => VisitTypeEnum::COMMON,
                'status' => ModelStatusEnum::OK
            ]);


            Website::query()->updateOrCreate([
                'name' => $name,
                'category_id' => $two->id,
                'organization_id' => $organization_id,
                'url' => $url,
                'type' => TypeEnum::ORGANIZATION,
                'status' => ModelStatusEnum::OK,
            ], [
                'description' => $des,
                'logo' => $filepath
            ]);

            $this->line($name);
            $this->line($logo_url);
            $i++;
        }

        //排序
        $categories = Category::query()->where('pid', 0)->where('organization_id', $organization_id)->orderBy('id')->pluck('id');
        $nums = count($categories);
        //一级
        foreach ($categories as $cid) {
            Category::query()->where('id', $cid)->update(['sort' => $nums]);
            $nums--;

            $ids = Category::query()->where('organization_id', $organization_id)->where('pid', $cid)->orderBy('id')->pluck('id');
            $nums_t = count($ids);
            foreach ($ids as $id) {
                Category::query()->where('id', $id)->update(['sort' => $nums_t]);
                $nums_t--;

                $ids_w = Website::query()->where('organization_id', $organization_id)->where('category_id', $id)->orderBy('id')->pluck('id');
                $nums_w = count($ids_w);
                foreach ($ids_w as $item) {
                    Website::query()->where('id', $item)->update(['sort' => $nums_w]);
                    $nums_w--;
                }
            }

        }
    }

}
