<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\Controllers\Api\Navigation;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\Navigation\RankTypeEnum;
use App\Repositories\Enums\Navigation\TypeEnum;
use App\Repositories\Enums\Navigation\VisitTypeEnum;
use App\Repositories\Models\CMS\Article;
use App\Repositories\Models\Navigation\Category;
use App\Repositories\Models\Navigation\Organization;
use App\Repositories\Models\Navigation\Rank;
use App\Repositories\Models\Navigation\SearchRecord;
use App\Repositories\Models\Navigation\Tag;
use App\Repositories\Models\Navigation\UserOrganization;
use App\Repositories\Models\Navigation\UserRecord;
use App\Repositories\Models\Navigation\Website;
use App\Repositories\Models\User\User;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;

class HomeController extends Controller
{
    /**
     * 首页
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function index(Request $request)
    {
        $key = $request->get('key', false);
        $url = $request->get('url', false);

        $organization_id = false;
        if ($key) {
            $organization_id = Organization::query()->where('slug', $key)->where('status', ModelStatusEnum::OK)->value('id');
        }

        if (!$organization_id && $url) {
            $url = parse_url($url);
            $host = isset($url['host']) ? $url['host'] : false;
            if ($host) {
                $prefix = str_replace(config('site.prefixUrl'), '', $host);
                $organization_id = Organization::query()->where('prefix', $prefix)->where('status', ModelStatusEnum::OK)->value('id');
            }
        }

        if (!$organization_id) {
//            $organization_id = Organization::query()->where('status', ModelStatusEnum::OK)->value('id');
            $organization_id = 39;
        }

        $user = false;
        if (is_user_login()) {
            $user = User::query()->where('id', login_user_id())->first();
        }

        $ranks = [];

        $organization = Organization::query()->where('id', $organization_id)->select(['id', 'prefix', 'name', 'logo', 'setting', 'background', 'pid', 'intro_title', 'intro', 'contact_code', 'is_rank'])->first();
        $organization['logo'] = path_to_url($organization['logo']);
        $organization['background'] = path_to_url($organization['background']);
        $host = 'http://' . $organization['prefix'] . config('site.prefixUrl');


        $is_rank_status = $organization->is_rank;

        //是否是该组织的成员
        $is_self = true;
        $role = 0;
        if ($user) {
            $role = UserOrganization::isMeOrganization($user->id, $organization_id);
            if ($role) {
                $is_self = false;
            }
        }

        $category_fields = ['id', 'name', 'type', 'logo', 'organization_id', 'visit_type', 'setting'];
        $categories = [];

        //继承父级组织的分类
        if ($organization->pid) {
            if ($user) {
                $pids = Organization::byIdGetParentsIds($organization_id);
                $organization_inherit_categories = Category::query()->status()->where('visit_type', '=', VisitTypeEnum::INHERIT)->where('pid', 0)->whereIn('organization_id', $pids)->where('type', TypeEnum::ORGANIZATION)->select($category_fields)->orderByDesc('sort')->orderByDesc('id')->get();
                $categories = array_merge($categories, $organization_inherit_categories->toArray());
            } else {
                $pids = Organization::byIdGetParentsIds($organization_id);
                $organization_inherit_categories = Category::query()->status()->where('visit_type', '=', VisitTypeEnum::INHERIT)->where('pid', 0)->where('hidden_status', 0)->whereIn('organization_id', $pids)->where('type', TypeEnum::ORGANIZATION)->select($category_fields)->orderByDesc('sort')->orderByDesc('id')->get();
                $categories = array_merge($categories, $organization_inherit_categories->toArray());
            }
        }

        //组织分类
        $organization_categories = Category::query()->status()->when($is_self, function ($query) {
            return $query->whereIn('visit_type', [VisitTypeEnum::COMMON, VisitTypeEnum::INHERIT])->where('hidden_status', 0);
        })->where('organization_id', $organization->id)->where('pid', 0)->where('type', TypeEnum::ORGANIZATION)->select($category_fields)->orderByDesc('sort')->orderByDesc('id')->get();
        $categories = array_merge($categories, $organization_categories->toArray());

        //个人
        if ($user) {
            $user_categories = Category::query()->status()->where('type', TypeEnum::PERSON)->where('pid', 0)->where('user_id', $user->id)->select($category_fields)->orderByDesc('sort')->orderByDesc('id')->get();
            $categories = array_merge($categories, $user_categories->toArray());
            $ranks = Rank::query()->where('key', $organization_id)->where('type', RankTypeEnum::CATEGORY_ONE)->where('user_id', $user->id)->orderBy('sort')->pluck('sort', 'type_id')->toArray();
        }

        //公共分类
//        $common_categories = Category::query()->status()->where('visit_type', '=', VisitTypeEnum::COMMON)->where('type', TypeEnum::COMMON)->where('organization_id', $organization->id)->where('pid', 0)->select($category_fields)->orderByDesc('sort')->orderByDesc('id')->get();
//        $categories = array_merge($categories, $common_categories->toArray());

        //站点
        $website_categories = Category::query()->status()->where('type', TypeEnum::WEBSITE)->where('organization_id', $organization->id)->where('pid', 0)->select($category_fields)->orderByDesc('sort')->orderByDesc('id')->get();
        $categories = array_merge($categories, $website_categories->toArray());

        //网址
        $websites = [];
        $sort = [];

        foreach ($categories as $k => $category) {
            $is_rank_status_user = $is_rank_status;
            if ($category['type'] == TypeEnum::PERSON) {
                $is_rank_status_user = true;
            }
            if (in_array($category['type'], [TypeEnum::COMMON, TypeEnum::ORGANIZATION, TypeEnum::PERSON])) {
                $children = [];
                //获取二级分类
                if ($category['type'] == TypeEnum::ORGANIZATION) {
                    if ($category['visit_type'] == VisitTypeEnum::INHERIT && $user) {
                        if ($organization_id == $category['organization_id']) {
                            $children = Category::query()->where('pid', $category['id'])->select($category_fields)->status()->orderByDesc('sort')->orderByDesc('id')->get();
                        } else {
                            //继承父类
                            $children = Category::query()->whereIn('visit_type', [VisitTypeEnum::INHERIT, VisitTypeEnum::COMMON])->where('pid', $category['id'])->select($category_fields)->status()->orderByDesc('sort')->orderByDesc('id')->get();
                        }
                    } else {
                        $children = Category::query()->when($is_self, function ($query) {
                            return $query->whereIn('visit_type', [VisitTypeEnum::COMMON, VisitTypeEnum::INHERIT]);
                        })->where('hidden_status', 0)->where('pid', $category['id'])->select($category_fields)->status()->orderByDesc('sort')->orderByDesc('id')->get();
                    }
                } else if ($category['type'] == TypeEnum::COMMON) {
                    $children = Category::query()->where('visit_type', '<>', VisitTypeEnum::OWNED)->where('pid', $category['id'])->select($category_fields)->status()->orderByDesc('sort')->orderByDesc('id')->get();
                } else if ($category['type'] == TypeEnum::PERSON) {
                    $children = Category::query()->where('pid', $category['id'])->select($category_fields)->status()->orderByDesc('sort')->orderByDesc('id')->get();
                }

                if (count($children)) {
                    $category_ranks = [];
                    if ($user && $is_rank_status) {
                        $category_ranks = Rank::query()->where('key', $category['id'])->where('type', RankTypeEnum::CATEGORY_TWO)->where('user_id', $user->id)->orderBy('sort')->pluck('sort', 'type_id')->toArray();
                    }
                    //获取二级分类网址
                    $children = $children->toArray();

                    foreach ($children as $i => $child) {
                        $website = Website::query()->when($is_self, function ($query) {
                            return $query->where('hidden_status', 0);
                        })->where('category_id', $child['id'])->select(['id', 'name', 'url', 'tags', 'description', 'logo'])->status()->orderByDesc('sort')->orderByDesc('id')->get()->toArray();

                        $website_ranks = [];
                        if ($user && $is_rank_status) {
                            $website_ranks = Rank::query()->where('key', $child['id'])->where('type', RankTypeEnum::WEBSITE)->where('user_id', $user->id)->orderBy('sort')->pluck('sort', 'type_id')->toArray();
                        }

                        foreach ($website as $key => $item) {
                            if ($item['tags']) {
                                $website[$key]['tags'] = Tag::query()->whereIn('id', str2arr($item['tags'], '-'))->pluck('name');
                            } else {
                                $website[$key]['tags'] = [];
                            }
                            if ($is_rank_status && count($website_ranks)) {
                                $website[$key]['sort'] = array_key_exists($item['id'], $website_ranks) ? $website_ranks[$item['id']] : 0;
                            }
                        }

                        //网址排序
                        if ($is_rank_status_user && count($website_ranks)) {
                            array_multisort(array_column($website, 'sort'), SORT_ASC, $website);//排序，根据$status 排
                        }

                        $children[$i]['urls'] = $website;
                        $children[$i]['sort'] = array_key_exists($child['id'], $category_ranks) ? $category_ranks[$child['id']] : 0;;
                    }

                    if ($is_rank_status_user && count($category_ranks)) {
                        array_multisort(array_column($children, 'sort'), SORT_ASC, $children);//排序，根据$status 排
                    }

                } else {
                    //获取一级分类网址
                    $website = Website::query()->when($is_self, function ($query) {
                        return $query->where('hidden_status', 0);
                    })->where('category_id', $category['id'])->select(['id', 'name', 'tags', 'url', 'description', 'logo'])->status()->orderByDesc('sort')->orderByDesc('id')->get();
                    $website_ranks = [];
                    if ($user && $is_rank_status) {
                        $website_ranks = Rank::query()->where('key', $category['id'])->where('type', RankTypeEnum::WEBSITE)->where('user_id', $user->id)->orderBy('sort')->pluck('sort', 'type_id')->toArray();
                    }
                    $website = $website->toArray();
                    foreach ($website as $key => $item) {
                        if ($item['tags']) {
                            $website[$key]['tags'] = Tag::query()->whereIn('id', str2arr($item['tags'], '-'))->pluck('name');
                        } else {
                            $website[$key]['tags'] = [];
                        }

                        if ($is_rank_status_user && count($website_ranks)) {
                            $website[$key]['sort'] = array_key_exists($item['id'], $website_ranks) ? $website_ranks[$item['id']] : 0;
                        }
                    }
                    //网址排序
                    if ($is_rank_status_user && count($website_ranks)) {
                        array_multisort(array_column($website, 'sort'), SORT_ASC, $website);//排序，根据$status 排
                    }
                }

                $websites[$k] = [
                    'id' => $category['id'],
                    'type' => $category['type'],
                    'name' => $category['name'],
                    'categories' => $children,
                    'urls' => $website,
                    'sort' => array_key_exists($category['id'], $ranks) ? $ranks[$category['id']] : 0
                ];
            }

            $sort[$k] = array_key_exists($category['id'], $ranks) ? $ranks[$category['id']] : 0;
        }

        if ($is_rank_status && count($ranks)) {
            $sortKey1 = array_column($websites, 'sort'); //取出数组中status的一列，返回一维数组
            $sortKey2 = $sort; //取出数组中status的一列，返回一维数组
            array_multisort($sortKey1, SORT_ASC, $websites);//排序，根据$status 排
            array_multisort($sortKey2, SORT_ASC, $categories);//排序，根据$status 排
        }

        $tops = [];
        $setting = [];
        if ($user) {
            $setting = User::query()->where('id', $user->id)->value('setting');
            if (is_array($setting) && array_key_exists('tops', $setting)) {
                foreach ($setting['tops'] as $t) {
                    $w = Website::query()->where('id', $t)->select(['id', 'name', 'url', 'description'])->first();
                    if ($w) {
                        $tops[] = $w;
                    }
                }
            }
        }

        $articles = Article::query()->where('category_id', 1)->select(['id', 'title', 'body'])->orderByDesc('sort')->get()->toArray();

        if (optional($organization)->intro_title) {
            $articles = array_merge([['id' => 0, 'title' => optional($organization)->intro_title, 'body' => optional($organization)->intro]], $articles);
        }

        $contact_code = false;
        if (optional($organization)->contact_code) {
            $contact_code = path_to_url($organization->contact_code);
        }

        return Response::success(compact('setting', 'tops', 'websites', 'categories', 'user', 'organization', 'articles', 'contact_code', 'host', 'role'));
    }


    /**
     * 搜索记录
     * Author: Mead
     */
    public function search(Request $request)
    {
        $keyword = $request->get('keyword');
        $type = $request->get('type');
        $user_id = $request->get('user_id', 0);
        $organization_id = $request->get('organization_id');

        SearchRecord::query()->create([
            'keyword' => $keyword,
            'type' => $type,
            'organization_id' => $organization_id,
            'user_id' => $user_id,
        ]);
        return Response::success(null);
    }

    /**
     * 点击记录
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * Author: Mead
     */
    public function click(Request $request)
    {
        $user_id = $request->get('user_id', 0);
        $organization_id = $request->get('organization_id');
        $website_id = $request->get('website_id');
        $category_id = $request->get('category_id');

        UserRecord::query()->create([
            'website_id' => $website_id,
            'category_id' => $category_id,
            'organization_id' => $organization_id,
            'user_id' => $user_id,
        ]);

        Website::query()->where('id', $website_id)->increment('click_nums');

        return Response::success(null);
    }

    /**
     * 置顶
     * @param Request $request
     * Author: Mead
     */
    public function top(Request $request)
    {
        $this->validate($request, [
            'website_id' => 'required|integer'
        ]);
        $user = login_user();
        $website_id = $request->get('website_id');

        $setting = User::query()->where('id', $user->id)->value('setting');

        if (is_array($setting) && array_key_exists('tops', $setting)) {
            $tops = $setting['tops'];
            array_unshift($tops, $website_id);
        } else {
            $tops[] = $website_id;
        }

        $tops = array_unique($tops);
        $tops = array_slice($tops, 0, 5);

        $setting['tops'] = $tops;

        User::query()->where('id', $user->id)->update(['setting' => $setting]);

        return Response::success(null);
    }

    /**
     * 删除置顶
     * @param Request $request
     * @return \Illuminate\Http\JsonResponse|\Illuminate\Http\Resources\Json\Resource
     * @throws \Illuminate\Validation\ValidationException
     * Author: Mead
     */
    public function deleteTop(Request $request)
    {
        $this->validate($request, [
            'website_id' => 'required|integer'
        ]);
        $user = login_user();
        $website_id = $request->get('website_id');

        $setting = User::query()->where('id', $user->id)->value('setting');
        if (is_array($setting) && array_key_exists('tops', $setting)) {
            $setting['tops'] = array_diff($setting['tops'], [$website_id]);
        }

        User::query()->where('id', $user->id)->update(['setting' => $setting]);
        return Response::success(null);
    }

    /**
     * 收藏站点
     * Author: Mead
     */
    public function collection(Request $request)
    {
        $this->validate($request, [
            'website_id' => 'required|integer',
            'name' => 'required',
            'category_id' => 'required|integer'
        ]);

        $user = login_user();
        $website_id = $request->get('website_id');
        $category_id = $request->get('category_id');
        $name = $request->get('name');

        //添加站点
        $website = Website::query()->where('id', $website_id)->first();
        if (!$website) {
            return Response::fail('找不到该站点');
        }

        $model = Website::query()->create([
            'name' => $name,
            'description' => $website->description,
            'type' => TypeEnum::PERSON,
            'tags' => null,
            'category_id' => $category_id,
            'logo' => $website->logo,
            'url' => $website->url,
            'user_id' => $user->id,
            'status' => ModelStatusEnum::OK,
        ]);

        return Response::success($model);
    }

    /**
     * 分类排序
     * Author: Mead
     */
    public function sortCategory(Request $request)
    {
        $type = $request->get('type');
        $data = $request->get('data', []);
        $user_id = login_user_id();

        $key = 0;
        switch ($type) {
            case RankTypeEnum::CATEGORY_ONE:
                $key = Category::query()->where('type', TypeEnum::ORGANIZATION)->whereIn('id', $data)->value('organization_id');
                break;
            case RankTypeEnum::CATEGORY_TWO:
                $key = Category::query()->where('type', TypeEnum::ORGANIZATION)->whereIn('id', $data)->value('pid');
                break;
            case RankTypeEnum::WEBSITE:
                $key = Website::query()->whereIn('id', $data)->value('category_id');
                break;
            default:
                break;
        }

        if (!$key) {
            return Response::fail('排序保存失败');
        }

        foreach ($data as $k => $d) {
            $index = $k + 1;

            Rank::query()->updateOrCreate([
                'user_id' => $user_id,
                'type' => $type,
                'key' => $key,
                'type_id' => $d
            ], [
                'sort' => $index,
            ]);
        }

        return Response::success(null);
    }


    /**
     * 排序
     * Author: Mead
     */
    public function sortMore(Request $request)
    {
        $data = $request->get('data', []);

        foreach ($data as $item) {
            if (array_key_exists('type', $item) && array_key_exists('data', $item) && is_array($item['data'])) {
                $type = $item['type'];
                $da = $item['data'];
                $user_id = login_user_id();

                $key = 0;
                switch ($type) {
                    case RankTypeEnum::CATEGORY_ONE:
                        $key = Category::query()->where('type', TypeEnum::ORGANIZATION)->whereIn('id', $da)->value('organization_id');
                        break;
                    case RankTypeEnum::CATEGORY_TWO:
                        $key = Category::query()->where('type', TypeEnum::ORGANIZATION)->whereIn('id', $da)->value('pid');
                        break;
                    case RankTypeEnum::WEBSITE:
                        $key = Website::query()->whereIn('id', $da)->value('category_id');
                        break;
                    default:
                        break;
                }

                if ($key) {
                    foreach ($da as $k => $d) {
                        $index = $k + 1;

                        Rank::query()->updateOrCreate([
                            'user_id' => $user_id,
                            'type' => $type,
                            'key' => $key,
                            'type_id' => $d
                        ], [
                            'sort' => $index,
                        ]);
                    }
                }
            }
        }

        return Response::success(null);
    }

    /**
     * 排序管理
     * @param Request $request
     * Author: Mead
     */
    public function sortManagement(Request $request)
    {
        $type = $request->get('type');
        $data = $request->get('data', []);
        $organization_id = $request->get('organization_id');
        $user = login_user();

        if (!UserOrganization::isMeManagementOrganization($user->id, $organization_id)) {
            return Response::fail('您暂无权限!');
        }

        $where = [];
        $where[] = ['organization_id', '=', $organization_id];
        switch ($type) {
            case RankTypeEnum::CATEGORY_ONE:
                $where[] = ['pid', '=', 0];
                Rank::query()->where('type', RankTypeEnum::CATEGORY_ONE)->where('key', $organization_id)->delete();
                break;
            case RankTypeEnum::CATEGORY_TWO:
                break;
            case RankTypeEnum::WEBSITE:
                break;
            default:
                return Response::fail('类型不对!');
                break;
        }

        $nums = count($data);

        foreach ($data as $id) {
            if (in_array($type, [RankTypeEnum::CATEGORY_ONE, RankTypeEnum::CATEGORY_TWO])) {
                Category::query()->where($where)->where('id', $id)->status()->update(['sort' => $nums]);
            } else {
                Website::query()->where($where)->where('id', $id)->status()->update(['sort' => $nums]);;
            }
            $nums--;
        }

        return Response::success();
    }
}
