<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Providers;

use Carbon\Carbon;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\ServiceProvider;
use Overtrue\EasySms\EasySms;

class AppServiceProvider extends ServiceProvider
{
    /**
     * Register any application services.
     */
    public function register()
    {
    }

    /**
     * Bootstrap any application services.
     *
     * @return void
     */
    public function boot()
    {
        if (App::getLocale() == 'zh_CN') {

            Carbon::setLocale('zh');
        }

        $this->registerObservers(); // 注册观察者

        //短信
        $this->app->singleton('easy_sms', function ($app) {
            return (new  EasySms($app->config['sms']));
        });


        Validator::extend('mobile', function ($attribute, $value, $parameters) {
            return preg_match('/^1[3456789][0-9]{9}$/', $value);
        }, T('mobile'));

        //记录执行效率
        DB::listen(function ($query) {
            if ($query->time <= 1000) {
                return;
            }
            $location = collect(debug_backtrace())->filter(function ($trace) {
                if (!isset($trace['file'])) {
                    return [
                        'file' => '',
                        'line' => 0,
                    ];
                }
                return !str_contains($trace['file'], 'vendor/');
            })->first(); // grab the first element of non vendor/ calls

            if (!isset($location['file'])) {
                $location = [
                    'file' => '',
                    'line' => 0,
                ];
            }
//            $bindings = implode(", ", $query->bindings); // format the bindings as string
            try {
                if (count($query->bindings)) {
                    $sql = vsprintf(str_replace("?", "'%s'", str_replace("%", '%%', $query->sql)), $query->bindings);
                } else {
                    $sql = $query->sql;
                }
            } catch (\Exception $exception) {
                $bindings = implode(", ", $query->bindings);
                Log::info("
            ------ERROR------
            Sql: $query->sql
            Bindings: $bindings
            ------------
        ");
                Log::error($exception);
                return;
            }

            Log::info("
            ------------
            Sql: $sql
            Time: $query->time
            File: ${location['file']}
            Line: ${location['line']}
            ------------
        ");
        });
    }

    protected function registerObservers(): void
    {
    }
}
