<?php

namespace App\Repositories\Criteria\Navigation;

use App\Repositories\Enums\ModelStatusEnum;
use App\Repositories\Enums\Navigation\TypeEnum;
use App\Repositories\Enums\PermissionEnum;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Prettus\Repository\Contracts\CriteriaInterface;
use Prettus\Repository\Contracts\RepositoryInterface;

/**
 * Class CategoryCriteria.
 *
 * @package namespace App\Repositories\Criteria\Navigation;
 */
class CategoryCriteria implements CriteriaInterface
{
    /**
     * Apply criteria in query repository
     *
     * @param string $model
     * @param RepositoryInterface $repository
     *
     * @return mixed
     */
    public function __construct(Request $request = null)
    {
        if (is_null($request)) {
            $this->request = \request();
        } else {
            $this->request = $request;
        }
    }

    /**
     * Apply criteria in query repository
     *
     * @param string $model
     * @param RepositoryInterface $repository
     *
     * @return mixed
     */
    public function apply($model, RepositoryInterface $repository)
    {
        if ($this->request->filled('name')) {
            $name = $this->request->get('name');
            $model = $model->where('name', 'like', "%{$name}%");
        }

        if ($this->request->filled('status')) {
            $status = $this->request->get('status');
            $model = $model->where('status', '=', $status);
        }
//        else {
//            $model = $model->where('status', '=', ModelStatusEnum::OK);
//        }

        if ($this->request->filled('pid')) {
            $pid = $this->request->get('pid', false);
            $model = $model->where('pid', '=', $pid);
        }

        if ($this->request->filled('type')) {
            $type = $this->request->get('type', false);
            $model = $model->where('type', '=', $type);
        }

        if ($this->request->filled('visit_type')) {
            $visit_type = $this->request->get('visit_type', false);
            $model = $model->where('visit_type', '=', $visit_type);
        }


        if ($this->request->filled('tier')) {
            $tier = $this->request->get('tier', false);
            $model = $model->where('tier', '=', $tier);
        }

        if ($this->request->filled('user_id')) {
            $user_id = $this->request->get('user_id', false);
            $model = $model->where('user_id', '=', $user_id);
        }

        if ($this->request->filled('organization_id')) {
            $organization_id = $this->request->get('organization_id', false);
            $model = $model->where('organization_id', '=', $organization_id);
        }

        $model = $model->orderByDesc('id');

        if (is_admin_login()) {
            $model->whereIn('type', [TypeEnum::ORGANIZATION, TypeEnum::COMMON, TypeEnum::WEBSITE]);
        }

//        $allDatePermission = PermissionEnum::DATA_CATEGORIES()->name;
//        $user = Auth::user();
//        if (!$user->can($allDatePermission)) {
//            //没有全部数据权限
//            $organizationDatePermission = PermissionEnum::DATA_CATEGORY_ORGANIZATION()->name;
//            if ($user->can($organizationDatePermission)) {
//                $model = $model->whereIn('organization_id', $user->organization_id);
//                return $model;
//            }
//            $selfDatePermission = PermissionEnum::DATA_CATEGORY_ME()->name;
//            if ($user->can($selfDatePermission)) {
//                $model = $model->where('user_id', $user->id);
//                return $model;
//            }
//            $model = $model->where('id', 0);
//            return $model;
//        }

        return $model;
    }
}
