<?php

namespace App\Repositories\Criteria\Navigation;

use App\Repositories\Enums\ModelStatusEnum;
use Illuminate\Http\Request;
use Prettus\Repository\Contracts\CriteriaInterface;
use Prettus\Repository\Contracts\RepositoryInterface;

/**
 * Class OrganizationCriteria.
 *
 * @package namespace App\Repositories\Criteria\Navigation;
 */
class OrganizationCriteria implements CriteriaInterface
{
    /**
     * Apply criteria in query repository
     *
     * @param string $model
     * @param RepositoryInterface $repository
     *
     * @return mixed
     */
    public function __construct(Request $request = null)
    {
        if (is_null($request)) {
            $this->request = \request();
        } else {
            $this->request = $request;
        }
    }

    /**
     * Apply criteria in query repository
     *
     * @param string $model
     * @param RepositoryInterface $repository
     *
     * @return mixed
     */
    public function apply($model, RepositoryInterface $repository)
    {
        if ($this->request->filled('name')) {
            $name = $this->request->get('name');
            $model = $model->where('name', 'like', "%{$name}%");
        }

        if ($this->request->filled('status')) {
            $status = $this->request->get('status');
            $model = $model->where('status', '=', $status);
        }
//        else {
//            $model = $model->where('status', '=', ModelStatusEnum::OK);
//        }

        if ($this->request->filled('pid')) {
            $pid = $this->request->get('pid', false);
            $model = $model->where('pid', '=', $pid);
        }

        if ($this->request->filled('tier')) {
            $tier = $this->request->get('tier', false);
            $model = $model->where('tier', '=', $tier);
        }

        if ($this->request->filled('user_id')) {
            $user_id = $this->request->get('user_id', false);
            $model = $model->where('user_id', '=', $user_id);
        }

        $model = $model->orderByDesc('id');
        return $model;
    }
}
