<?php

namespace App\Repositories\Criteria\Navigation;

use App\Repositories\Enums\ModelStatusEnum;
use Illuminate\Http\Request;
use Prettus\Repository\Contracts\CriteriaInterface;
use Prettus\Repository\Contracts\RepositoryInterface;

/**
 * Class WebsiteCriteria.
 *
 * @package namespace App\Repositories\Criteria\Navigation;
 */
class WebsiteCriteria implements CriteriaInterface
{
    /**
     * Apply criteria in query repository
     *
     * @param string $model
     * @param RepositoryInterface $repository
     *
     * @return mixed
     */
    public function __construct(Request $request = null)
    {
        if (is_null($request)) {
            $this->request = \request();
        } else {
            $this->request = $request;
        }
    }

    /**
     * Apply criteria in query repository
     *
     * @param string $model
     * @param RepositoryInterface $repository
     *
     * @return mixed
     */
    public function apply($model, RepositoryInterface $repository)
    {
        if ($this->request->filled('name')) {
            $name = $this->request->get('name');
            $model = $model->where('name', 'like', "%{$name}%");
        }

        if ($this->request->filled('status')) {
            $status = $this->request->get('status');
            $model = $model->where('status', '=', $status);
        }
//        else {
//            $model = $model->where('status', '=', ModelStatusEnum::OK);
//        }

        if ($this->request->filled('description')) {
            $description = $this->request->get('description');
            $model = $model->where('description', 'like', $description);
        }

        if ($this->request->filled('type')) {
            $type = $this->request->get('type', false);
            $model = $model->where('type', '=', $type);
        }

        if ($this->request->filled('category_id')) {
            $category_id = $this->request->get('category_id', false);
            $model = $model->where('category_id', '=', $category_id);
        }

        if ($this->request->filled('organization_id')) {
            $organization_id = $this->request->get('organization_id', false);
            $model = $model->where('organization_id', '=', $organization_id);
        }

        if ($this->request->filled('user_id')) {
            $user_id = $this->request->get('user_id', false);
            $model = $model->where('user_id', '=', $user_id);
        }

        if ($this->request->filled('url')) {
            $url = $this->request->get('url');
            $model = $model->where('url', 'like', $url);
        }

        $model = $model->orderByDesc('id');
        return $model;
    }
}
