<?php

namespace App\Repositories\Models\Navigation;

use App\Repositories\Models\Model;
use App\Repositories\Models\User\User;
use Illuminate\Support\Facades\DB;
use Prettus\Repository\Contracts\Transformable;
use Prettus\Repository\Traits\TransformableTrait;
use function Sodium\compare;

/**
 * Class Category.
 *
 * @package namespace App\Repositories\Models\Navigation;
 */
class Category extends Model implements Transformable
{
    use TransformableTrait;

    protected $table = 'navigation_categories';

    protected $guarded = [];

    protected $casts = [
        'setting' => 'json'
    ];

    protected static function booted()
    {
        static::saved(function (Category $model) {
            if ($model->pid) {
                $nums = self::query()->where('pid', $model->pid)->status()->count();
                if ($nums === 1) {
                    Website::query()->where('category_id', $model->pid)->update([
                        'category_id' => $model->id,
                    ]);
                }
            }
        });
    }

    /**
     * 根据id获取层级
     * Author: Mead
     */
    public static function byIdGetTier($id)
    {
        if ($id == 0) {
            return 1;
        }

        $t = self::query()->where('id', $id)->value('tier');
        return ($t + 1);
    }

    public function user()
    {
        return $this->belongsTo(User::class)->select(['id', 'name']);
    }

    public function parent()
    {
        return $this->belongsTo(Organization::class, 'pid', 'id');
    }

    public function getLogoResourceAttribute()
    {
        return path_to_url($this->attributes['logo']);
    }

    public function organization()
    {
        return $this->belongsTo(Organization::class)->select(['id', 'name']);
    }

    public static $data = [];

    public static function trees($id, $where = ['select' => ['*']], $is_pid = true, $level = 1)
    {
        $model = self::query();
        if (!$is_pid) {
            $model->where('id', $id);
        } else {
            $model->where('pid', $id);
        }

        if (array_key_exists('where', $where)) {
            $model->where($where['where']);
        }

        if (array_key_exists('select', $where)) {
            $model->select($where['select']);
        }

        if (array_key_exists('sort', $where)) {
            $model->orderByRaw(DB::raw($where['sort']));
        }

        $ds = $model->get()->toArray();

        foreach ($ds as $k => $d) {
            $ds[$k]['level'] = $level;

//            if (array_key_exists('select', $where) && (in_array('*', $where['select']) || in_array('logo', $where['select']))) {
//                $ds[$k]['logo'] = path_to_url($d['logo']);
//            }

            if (self::query()->where('pid', $d['id'])->exists()) {
                $ds[$k]['child'] = self::trees($d['id'], $where, true, $level + 1);
            } else {
                $ds[$k]['child'] = [];
            }
        }
        return $ds;
    }

    private static $ppid = 0;

    public static function sortTrees($data, $pid = 0)
    {
        $arr = [];
        if (empty($data)) {
            return '';
        }

        foreach ($data as $key => $value) {
            $n_arr = [];
            if ($value['pid'] == $pid) {
                if ($pid == 0) {
                    $key = self::$ppid;
                    self::$ppid++;
                }
                $n_arr = $value;

                $n_arr['child'] = self::sortTrees($data, $value['id']);
                $arr[] = $n_arr;
            }
        }
        return $arr;
    }

    public static function isChild($id)
    {
        return self::query()->where('pid', $id)->status()->exists();
    }

}
