<?php

namespace App\Repositories\Models\Navigation;

use App\Repositories\Models\Model;
use App\Repositories\Models\User\User;
use Illuminate\Support\Facades\DB;
use Prettus\Repository\Contracts\Transformable;
use Prettus\Repository\Traits\TransformableTrait;

/**
 * Class Organization.
 *
 * @package namespace App\Repositories\Models\Navigation;
 */
class Organization extends Model implements Transformable
{
    use TransformableTrait;

    protected $table = 'navigation_organizations';

    protected $guarded = [];

    /**
     * 根据id获取层级
     * Author: Mead
     */
    public static function byIdGetTier($id)
    {
        if ($id == 0) {
            return 1;
        }

        $t = self::query()->where('id', $id)->value('tier');
        return ($t + 1);
    }

    public function user()
    {
        return $this->belongsTo(User::class)->select(['id', 'name']);
    }

    public function parent()
    {
        return $this->belongsTo(Organization::class, 'pid', 'id');
    }

    public function getLogoResourceAttribute()
    {
        return path_to_url($this->attributes['logo']);
    }

    public function getBackgroundResourceAttribute()
    {
        return path_to_url($this->attributes['background']);
    }

    public static $data = [];

    public static function trees($id, $where = ['select' => ['*']], $is_pid = true, $level = 1)
    {
        $model = self::query();
        if (!$is_pid) {
            $model->where('id', $id);
        } else {
            $model->where('pid', $id);
        }

        if (array_key_exists('where', $where)) {
            $model->where($where['where']);
        }

        if (array_key_exists('select', $where)) {
            $model->select($where['select']);
        }

        if (array_key_exists('sort', $where)) {
            $model->orderByRaw(DB::raw($where['sort']));
        }

        $ds = $model->get()->toArray();

        foreach ($ds as $k => $d) {
            $ds[$k]['level'] = $level;

            if (array_key_exists('select', $where) && (in_array('*', $where['select']) || in_array('logo', $where['select']))) {
                $ds[$k]['logo'] = path_to_url($d['logo']);
            }

            if (self::query()->where('pid', $d['id'])->exists()) {
                $ds[$k]['child'] = self::trees($d['id'], $where, true, $level + 1);
            } else {
                $ds[$k]['child'] = [];
            }
        }
        return $ds;
    }

    public static function isChild($id)
    {
        return self::query()->where('pid', $id)->status()->exists();
    }

    /**
     * 根据id获取父id
     * @param $id
     * Author: Mead
     */
    public static $pidData = [];

    public static function byIdGetParentsIds($id, $isRoot = true)
    {
        $pid = Organization::query()->where('id', $id)->value('pid');
        if ($pid) {
            self::$pidData[] = $pid;
            self::byIdGetParentsIds($pid, false);
        }

        if ($isRoot) {
            return self::$pidData;
        }
    }
}
