<?php

namespace App\Repositories\Models\Navigation;

use App\Repositories\Models\Model;
use App\Repositories\Models\User\User;
use Illuminate\Support\Facades\DB;
use Prettus\Repository\Contracts\Transformable;
use Prettus\Repository\Traits\TransformableTrait;

/**
 * Class Website.
 *
 * @package namespace App\Repositories\Models\Navigation;
 */
class Website extends Model implements Transformable
{
    use TransformableTrait;

    protected $table = 'navigation_websites';

    protected $guarded = [];

    /**
     * 根据id获取层级
     * Author: Mead
     */
    public static function byIdGetTier($id)
    {
        if ($id == 0) {
            return 1;
        }

        $t = self::query()->where('id', $id)->value('tier');
        return ($t + 1);
    }

    public function user()
    {
        return $this->belongsTo(User::class)->select(['id', 'name']);
    }

    public function getLogoResourceAttribute()
    {
        return path_to_url($this->attributes['logo']);
    }

    public function getLogoAttribute()
    {
        return path_to_url($this->attributes['logo']);
    }

    public function setTagsAttribute($tags)
    {
        if (is_array($tags)) {
            $this->attributes['tags'] = '-' . arr2str0($tags, '-') . '-';
        } else {
            $this->attributes['tags'] = $tags;
        }

    }

    public static function getTags($tags)
    {
        $ids = str2arr0(trim($tags, '-'), '-');
        return Tag::query()->whereIn('id', $ids)->select(['id', 'name'])->get()->toArray();

    }

    public function organization()
    {
        return $this->belongsTo(Organization::class)->select(['id', 'name']);
    }

    public function category()
    {
        return $this->belongsTo(Category::class)->select(['id', 'name']);
    }

    public static $data = [];

    public static function trees($id, $where = ['select' => ['*']], $is_pid = true, $level = 1)
    {
        $model = self::query();
        if (!$is_pid) {
            $model->where('id', $id);
        } else {
            $model->where('pid', $id);
        }

        if (array_key_exists('where', $where)) {
            $model->where($where['where']);
        }

        if (array_key_exists('select', $where)) {
            $model->select($where['select']);
        }

        if (array_key_exists('sort', $where)) {
            $model->orderByRaw(DB::raw($where['sort']));
        }

        $ds = $model->get()->toArray();

        foreach ($ds as $k => $d) {
            $ds[$k]['level'] = $level;

            if (array_key_exists('select', $where) && in_array('logo', $where['select'])) {
                $ds[$k]['logo'] = path_to_url($d['logo']);
            }

            if (self::query()->where('pid', $d['id'])->exists()) {
                $ds[$k]['child'] = self::trees($d['id'], $where, true, $level + 1);
            } else {
                $ds[$k]['child'] = [];
            }
        }
        return $ds;
    }

}
