<?php

namespace App\Repositories\Models\User;

use App\Repositories\Enums\RoleEnum;
use App\Repositories\Models\Course\OrganizationStudent;
use App\Repositories\Models\Model;
use App\Repositories\Models\Navigation\Organization;
use Database\Factories\UserFactory;
use Illuminate\Auth\Authenticatable;
use Illuminate\Contracts\Auth\Access\Authorizable as AuthorizableContract;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Laravel\Lumen\Auth\Authorizable;
use Spatie\Permission\Traits\HasRoles;
use Tymon\JWTAuth\Contracts\JWTSubject;

/**
 * Class Student.
 *
 * @package namespace App\Repositories\Models\User;
 */
class User extends Model implements AuthenticatableContract, AuthorizableContract, JWTSubject
{
    use Authenticatable, Authorizable, HasFactory, HasRoles;

    protected $table = 'base_users';

    const ROLE_ORGANIZATION_ADMIN_NAME = '组织管理员';
    const ROLE_USER = '用户';

    protected $casts = [
        'setting' => 'json'
    ];

    /**
     * The attributes that are mass assignable.
     *
     * @var array
     */
    protected $guarded = [];

    protected $hidden = ['password'];

    /**
     * 兼容 Laravel 8 的 Factory.
     *
     * @return UserFactory
     */
    protected static function newFactory()
    {
        return UserFactory::new();
    }

    /**
     * Get the identifier that will be stored in the subject claim of the JWT.
     *
     * @return mixed
     */
    public function getJWTIdentifier()
    {
        return $this->getKey();
    }

    /**
     * Return a key value array, containing any custom claims to be added to the JWT.
     *
     * @return array
     */
    public function getJWTCustomClaims()
    {
//        if (array_key_exists('role_id', $this->attributes) && $this->attributes['role_id']) {
//            //老师
//            return ['role' => 'teacher'];
//        }
//        //学生
//        return ['role' => 'user'];
        return ['role' => 'user'];
//        return [];
    }

    /**
     * 组织
     * Author: Mead
     */
    public function organization()
    {
        return $this->belongsTo(Organization::class, 'organization_id', 'id')->select(['id', 'name']);
    }

//    public function getOrganizationAttribute()
//    {
//        if ($organization_id = request('organization_id')) {
//            return OrganizationStudent::query()->where('user_id', $this->attributes['id'])->where('organization_id', $organization_id)->status()->select(['type'])->first();
//        }
//        return [];
//    }

    public function getHeadimgAttribute($val)
    {
        if (is_null($val)) {
            return path_to_url('default/headImg.jpeg');
        }
        return path_to_url($val);
    }


    /**
     * 权限
     * @return bool
     * Author: Mead
     */
    public function isSuperAdmin()
    {
        return $this->hasRole(RoleEnum::SUPER_ADMIN()->name);
    }

    public function isOwner($model)
    {
        if ($model->user_id === $this->id) {
            return true;
        }
        return false;
    }

    public function isOwnerOrganization($model)
    {
        if ($model->organization_id === $this->organization_id) {
            return true;
        }
        return false;
    }

}
