<?php

namespace App\Repositories\Transformers\Navigation;

use App\Repositories\Enums\Navigation\TypeEnum;
use App\Repositories\Models\Navigation\Category;
use App\Repositories\Transformers\BaseTransformer;
use Carbon\Carbon;

/**
 * Class CategoryTransformer.
 *
 * @package namespace App\Repositories\Transformers\Navigation;
 */
class CategoryTransformer extends BaseTransformer
{
    /**
     * Transform the Category entity.
     *
     * @param \App\Repositories\Models\Navigation\Category $model
     *
     * @return array
     */
    public function lists($model)
    {
        return [
            'id' => (int)$model->id,
            'name' => $model->name,
            'pid' => $model->pid,
            'tier' => $model->tier,
            'type' => $model->type,
            'type_text' => TypeEnum::getDescription($model->type),
            'logo' => $model->logo,
            'isChild' => Category::isChild($model->id),
            'organization' => $model->organization,
            'visit_type' => $model->visit_type,
            'hidden_status' => $model->hidden_status,
            'slug' => $model->slug,
            'sort' => $model->sort,
            'user' => $model->user,
            'status' => $model->status,
            'updated_at' => $model->updated_at->format(Carbon::DEFAULT_TO_STRING_FORMAT)
        ];
    }

    public function detail($model)
    {
        return [
            'id' => (int)$model->id,
            'name' => $model->name,
            'pid' => $model->pid,
            'type' => $model->type,
            'type_text' => TypeEnum::getDescription($model->type),
            'tier' => $model->tier,
            'logo' => $model->logo,
//            'logo_resource' => $model->logo_resource,
            'organization' => $model->organization,
            'visit_type' => $model->visit_type,
            'hidden_status' => $model->hidden_status,
            'slug' => $model->slug,
            'user' => $model->user,
            'sort' => $model->sort,
            'setting' => $model->setting,
            'status' => $model->status,
            'created_at' => $model->created_at->format(Carbon::DEFAULT_TO_STRING_FORMAT),
            'updated_at' => $model->updated_at->format(Carbon::DEFAULT_TO_STRING_FORMAT)
        ];
    }
}
